<?php

namespace Drupal\search_api_gendev\Controller;

use Drupal\Component\Utility\Timer;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\devel\Controller\EntityDebugController;
use Drupal\devel\DevelDumperManagerInterface;
use Drupal\search_api\Item\ItemInterface;
use Drupal\search_api\Utility\Utility;
use Drupal\search_api_gendev\Form\ItemForm;
use Drupal\search_api_gendev\SearchApiGendev;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for devel search api.
 */
class ItemController extends EntityDebugController {

  /**
   * Search API Gendev service instance.
   *
   * @var \Drupal\search_api_gendev\SearchApiGendev
   */
  protected $searchApiGendev;

  /**
   * ItemController constructor.
   *
   * @param \Drupal\devel\DevelDumperManagerInterface $dumper
   *   The dumper service.
   * @param \Drupal\search_api_gendev\SearchApiGendev $search_api_gendev
   *   The Search API Gendev service.
   */
  public function __construct(DevelDumperManagerInterface $dumper, SearchApiGendev $search_api_gendev) {
    $this->dumper = $dumper;
    $this->searchApiGendev = $search_api_gendev;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('devel.dumper'),
      $container->get('search_api_gendev')
    );
  }

  /**
   * View search_api entity data.
   *
   * @todo Do something about the UI.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   A RouteMatch object.
   *
   * @return array
   *   Array of page elements to render.
   */
  public function entitySearchApi(RouteMatchInterface $route_match) {
    $output = [];

    $entity = $this->getEntityFromRouteMatch($route_match);
    $entity_type_id = $entity->getEntityTypeId();

    if ($entity instanceof EntityInterface) {
      /** @var \Drupal\search_api\IndexInterface[] */
      $indices = $this->entityTypeManager()->getStorage('search_api_index')->loadByProperties(['status' => TRUE]);

      foreach ($indices as $index) {
        if (!empty($output)) {
          $output[] = ['#type' => 'html_tag', '#tag' => 'hr'];
        }

        $output[] = [
          '#type' => 'html_tag',
          '#tag' => 'h2',
          '#value' => $this->t('Index: %index', ['%index' => $index->toLink()->toString()]),
        ];

        foreach ($index->getDatasources() as $datasource) {
          if ($datasource->getEntityTypeId() !== $entity_type_id) {
            continue;
          }

          $item_id = Utility::createCombinedId($datasource->getPluginId(), $datasource->getItemId($entity->getTypedData()));

          Timer::start('search_api_gendev__query_index');
          $result_set = $index->query()
            ->addCondition('search_api_id', $item_id)
            ->execute();
          $time = Timer::stop('search_api_gendev__query_index');

          $output[] = [
            '#type' => 'html_tag',
            '#tag' => 'h3',
            '#value' => $this->t('Item: %item (query time: @ms ms)', [
              '%item' => $item_id,
              '@ms' => $time['time'],
            ]),
          ];

          $output[] = \Drupal::formBuilder()->getForm(ItemForm::class, $entity, $datasource);

          $output[] = [
            '#type' => 'html_tag',
            '#tag' => 'h4',
            '#value' => $this->t('Indexed data'),
          ];

          if ($result_set->getResultCount() > 0) {
            foreach ($result_set->getResultItems() as $item) {
              if ($item->getId() !== $item_id) {
                continue;
              }

              $output[] = $this->getItemOutput($item);
            }
          }
          else {
            $output[] = ['#markup' => $this->t('Not indexed')];
          }

          Timer::start('search_api_gendev__prepare_items');
          $items = $this->searchApiGendev->prepareItems($index, [$item_id]);
          $time = Timer::stop('search_api_gendev__prepare_items');

          $output[] = [
            '#type' => 'html_tag',
            '#tag' => 'h4',
            '#value' => $this->t('Local data (prepare time: @ms ms)', ['@ms' => $time['time']]),
          ];
          $output[] = $this->getItemOutput(reset($items));
        }
      }
    }

    return $output;
  }

  /**
   * Create the debugging output from an Item.
   */
  protected function getItemOutput(ItemInterface $item) {
    $fields = [];
    foreach ($item->getFields() as $field) {
      $fields[$field->getPropertyPath()] = $field->getValues();
    }

    ksort($fields);

    $out = $this->dumper->exportAsRenderable($fields);

    if ($extra_data = $item->getAllExtraData()) {
      $out[] = $this->dumper->exportAsRenderable($extra_data);
    }

    return $out;
  }

}
